import base64, codecs, json, logging, os
from hashlib import sha256
from typing import Any, Union

LoggerType = logging.Logger


### FIX ME ### can be more specific about response arg's type... somehow


def log_response(response: Any) -> str:
    """
    renders a python-requests response as json or as a string
    """
    try:
        log_body = response.json()
    except ValueError:
        log_body = response.content[:40]
    return log_body


def create_logger(name: str, log_level: Union[str, int]) -> LoggerType:
    """
    return a logger configured with name and log_level
    """

    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    if not logger.hasHandlers():
        handler = logging.StreamHandler()
        formatter = logging.Formatter("%(message)s")
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    return logger


def safe_base64(un_encoded_data: Union[str, bytes]) -> str:
    "return ACME-safe base64 encoding of un_encoded_data as a string"

    if isinstance(un_encoded_data, str):
        un_encoded_data = un_encoded_data.encode("utf8")
    r = base64.urlsafe_b64encode(un_encoded_data).rstrip(b"=")
    return r.decode("utf8")


def dns_challenge(key_auth: str) -> str:
    "return the ACME challenge response for a DNS TXT record"

    return safe_base64(sha256(key_auth.encode("utf8")).digest())


_sewer_meta = None


def sewer_meta(name: str) -> str:
    """
    returns the named attribute from lazily-loaded  meta.json (replaces __version__.py)
    """

    global _sewer_meta

    if _sewer_meta is None:
        here = os.path.abspath(os.path.dirname(__file__))
        with codecs.open(os.path.join(here, "meta.json"), "r", encoding="utf8") as f:
            _sewer_meta = json.load(f)
    return _sewer_meta[name]



import codecs, importlib, json, os, sys
from typing import Dict, List, Sequence

from .auth import ProviderBase


class ProviderDescriptor:
    def __init__(
        self,
        *,
        name: str,
        desc: str,
        chals: Sequence[str],
        args: Sequence[Dict[str, str]],
        deps: Sequence[str],
        path: str = None,
        cls: str = None,
        features: Sequence[str] = None,
        memo: str = None
    ) -> None:
        "initialize a driver descriptor from one item in the catalog"

        self.name = name
        self.desc = desc
        self.chals = chals
        self.args = args
        self.deps = deps
        self.path = path
        self.cls = cls
        self.features = [] if features is None else features
        self.memo = memo

    def __str__(self) -> str:
        return "Descriptor %s" % self.name

    def get_provider(self) -> ProviderBase:
        "return the class that implements this driver"

        module_name = self.path if self.path else ("sewer.providers." + self.name)
        module = importlib.import_module(module_name)
        return getattr(module, self.cls if self.cls else "Provider")


class ProviderCatalog:
    def __init__(self, filepath: str = "") -> None:
        "intialize a catalog from either the default catalog.json or one named by filepath"

        if not filepath:
            here = os.path.abspath(os.path.dirname(__file__))
            filepath = os.path.join(here, "catalog.json")
        with codecs.open(filepath, "r", encoding="utf8") as f:
            raw_catalog = json.load(f)

        items: Dict[str, ProviderDescriptor] = {}
        for item in raw_catalog:
            k = item["name"]
            if k in items:
                print("WARNING: duplicate name %s skipped in catalog %s" % (k, filepath))
            else:
                items[k] = ProviderDescriptor(**item)
        self.items = items

    def get_item_list(self) -> List[ProviderDescriptor]:
        "return the list of items in the catalog, sorted by name"

        res = [i for i in self.items.values()]
        res.sort(key=lambda i: i.name)
        return res

    def get_descriptor(self, name: str) -> ProviderDescriptor:
        "return the ProviderDescriptor that matches name"

        return self.items[name]

    def get_provider(self, name: str) -> ProviderBase:
        "return the class that implements the named driver"

        return self.get_descriptor(name).get_provider()



import subprocess

from .common import BaseDns


class UnboundSsh(BaseDns):
    """
    Working demo of using aliasing with legacy DNS model.

    This punts the authorization issue to the ssh command.  For unattended
    operation you'd need to run this with the key preloaded in ssh-agent or
    some equivalent.

    CLI options with alias (requires latest pre-0.8.3 code):
        --provider unbound_ssh --p_opts alias=... ssh_des=user@host

    Client usage:
        provider = UnboundSsh(ssh_des="user@host", alias="...")
        acme = client.Client(domain="host.your.domain", provider=provider, ...)
        certificate = acme.cert()
        ...
    """

    def __init__(self, *, ssh_des, **kwargs):
        """
        ssh_des is a REQUIRED keyword option that specifies the "destination"
        argument for making the SSH connection (see ssh(1)).  It is ASSUMED
        that the remote login has access to unbound's key so that it can run
        unbound-control to update the alias zone.
        """

        super().__init__(**kwargs)
        self.ssh_des = ssh_des

    def create_dns_record(self, host_fqdn, acme_challenge):
        self.manage_dns_record(host_fqdn, acme_challenge, "add")

    def delete_dns_record(self, host_fqdn, acme_challenge):
        self.manage_dns_record(host_fqdn, acme_challenge, "del")

    def manage_dns_record(self, host_fqdn, acme_challenge, cmd):
        """
        The first line of code here is really the whole of the aliasing demo - everything
        else is just the scaffolding to make this work in my quirky environment.  :-)

        NB: faking the challenge dict like this is potentially fragile.  Much better to
        migrate the legacy DNS driver to the new model if possible!
        """

        fqdn = self.target_domain({"domain": host_fqdn})

        update_cmd = unbound_command(cmd, fqdn, acme_challenge)
        res = subprocess.run(("ssh", self.ssh_des, "unbound-control -- ", update_cmd))
        if res.returncode != 0:
            raise RuntimeError(
                "FAILURE (%s): unbound command failed:\n  %s" % (res.returncode, res.args)
            )


# DO # NOT # USE # @classmethod # just to hide the function in the class.  Namespaces are great!


def unbound_command(cmd, fqdn, acme_challenge):
    if cmd == "add":
        return "local_data %s. 300 IN TXT '%s'" % (fqdn, acme_challenge)
    if cmd == "del":
        return "local_data_remove %s." % (fqdn,)

    raise ValueError("Unrecognized command to unbound_command: %s" % (cmd,))



from .lib import create_logger, LoggerType
from typing import Any, Dict, Optional, Sequence, Tuple, Union, cast

ChalItemType = Dict[str, str]
ChalListType = Sequence[ChalItemType]
ErrataItemType = Tuple[str, str, ChalItemType]
ErrataListType = Sequence[ErrataItemType]


class ProviderBase:
    """
    New-model driver documentation is in docs/UnifiedProvider.md
    """

    def __init__(
        self,
        *,
        chal_types: Sequence[str],
        logger: Optional[LoggerType] = None,
        LOG_LEVEL: str = "INFO",
        prop_delay: int = 0,
        prop_timeout: int = 0,
        prop_sleep_times: Union[Sequence[int], int] = (1, 2, 4, 8)
    ) -> None:

        # TypeError if missing, still check that it's a sequencey value; non-str vals, meh
        if not isinstance(chal_types, (list, tuple)):
            raise ValueError("chal_types must be a list or tuple of strings, not: %s" % chal_types)
        self.chal_types = chal_types

        # setup logging.  let it pass if both are given; logger supersedes old LOG_LEVEL
        if logger:
            self.logger = logger
        else:
            self.logger = create_logger(__name__, LOG_LEVEL)

        # prop_* control delay before and timeout of checking loop as well as internal sleeps
        self.prop_delay = int(prop_delay)
        self.prop_timeout = int(prop_timeout)

        ### eratta ### accepts str value(s) that pass int(); low importance
        if isinstance(prop_sleep_times, (list, tuple)):
            self.prop_sleep_times = tuple(int(v) for v in prop_sleep_times)
        else:
            self.prop_sleep_times = (int(cast(int, prop_sleep_times)),)

    def setup(self, challenges: ChalListType) -> ErrataListType:
        raise NotImplementedError("setup method not implemented by %s" % self.__class__)

    def unpropagated(self, challenges: ChalListType) -> ErrataListType:
        raise NotImplementedError("unpropagated method not implemented by %s" % self.__class__)

    def clear(self, challenges: ChalListType) -> ErrataListType:
        raise NotImplementedError("clear method not implemented by %s" % self.__class__)


class HTTPProviderBase(ProviderBase):
    """
    Base class for new-model HTTP drivers

    Currently this is a null adapter, holding a place in line for any future shared
    implementation.  It may never become non-null aside from the default chal_types
    it provides, but it's a small price to pay to avoid having to stuff it in later.
    """

    def __init__(self, **kwargs: Any) -> None:
        if "chal_types" not in kwargs:
            kwargs["chal_types"] = ["http-01"]
        super().__init__(**kwargs)


class DNSProviderBase(ProviderBase):
    """
    Base class for new-model DNS drivers - legacy drivers use the one in common.py

    Accepts the alias optional argument and adds cname_domain and target_domain
    to support the implementation of aliasing in drivers that inherit from it.
    """

    def __init__(self, *, alias: str = "", **kwargs: Any) -> None:
        if "chal_types" not in kwargs:
            kwargs["chal_types"] = ["dns-01"]
        super().__init__(**kwargs)
        self.alias = alias

    ### support for using a DNS alias

    def cname_domain(self, chal: Dict[str, str]) -> Union[str, None]:
        "returns fqdn where CNAME should be if aliasing, else None"

        return "_acme-challenge." + chal["domain"] if self.alias else None

    def target_domain(self, chal: Dict[str, str]) -> str:
        "returns fqdn where challenge TXT should be placed"

        d = chal["domain"]
        return "_acme-challenge." + d if not self.alias else d + "." + self.alias



import argparse, os
from typing import List

from .catalog import ProviderCatalog
from . import client, config, lib


def setup_parser(catalog):
    """
    return configured ArgumentParser - catalog-driven list of providers
    """

    parser = argparse.ArgumentParser(
        prog="sewer",
        description="Sewer is an ACME client for getting certificates from Let's Encrypt",
        allow_abbrev=False,
        formatter_class=argparse.RawTextHelpFormatter,
    )

    ### immediate action "options"

    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=lib.sewer_meta("version")),
        help="The currently installed sewer version.",
    )
    parser.add_argument(
        "--known_providers",
        action="version",
        version="Known Providers:\n    "
        + "\n    ".join("%s  %s" % (i.name, i.desc) for i in catalog.get_item_list()),
        help="Show a list of the known providers and exit.",
    )

    ### ACME account options

    parser.add_argument(
        "--account_key",
        type=argparse.FileType("r"),
        help="Filepath to read to get registered ACME account.  Default is to create one.",
    )
    parser.add_argument("--email", help="Email to be used for registration of an ACME account.")

    ### certificate options

    parser.add_argument(
        "--certificate_key",
        type=argparse.FileType("r"),
        help="Filepath to read to get certificate key.  Default is to create one.",
    )
    parser.add_argument(
        "--domain",
        required=True,
        help="The DNS identity which will be the certificate's Common Name.  May be a wildcard.",
    )
    parser.add_argument(
        "--alt_domains",
        default=[],
        nargs="*",
        help="Optional alternate (SAN) identities to be added to the CN on this certificate.",
    )
    parser.add_argument(
        "--bundle_name",
        help="The basename for the output files.  Default is the CN given by --domain.",
    )
    parser.add_argument(
        "--out_dir",
        default=os.getcwd(),
        help="Directory that stores certificate and keys files; current dir is default.",
    )

    ### challenge provider options

    parser.add_argument(
        "--provider",
        "--dns",
        metavar="<name>",
        dest="provider",
        required=True,
        choices=[i.name for i in catalog.get_item_list()],
        help="Name of the challenge provider to use.  (--dns is OBSOLESCENT; prefer --provider)",
    )
    parser.add_argument(
        "--p_opts", default=[], nargs="*", help="Option(s) to pass to provider, each is key=value"
    )

    ### protocol options

    parser.add_argument(
        "--endpoint",
        default="production",
        choices=["production", "staging"],
        help="Select between Let's Encrypt's endpoints.  Default is production.",
    )
    parser.add_argument(
        "--acme_timeout",
        type=int,
        default=7,
        help="The maximum time the client will wait for a network call (HTTPS request to ACME server) to complete.  Default is 7",
    )

    ### sewer command options

    parser.add_argument(
        "--loglevel",
        default="INFO",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        help="The log level to output log messages at. \
        eg: --loglevel DEBUG",
    )
    parser.add_argument(
        "--action",
        choices=["run", "renew"],
        default="none",
        help="[DEPRECATED] The action that you want to perform (has never done anything).",
    )

    return parser


def get_provider(provider_name, provider_kwargs, catalog, logger):
    """
    return class (or callable) that will return the Provider instance to use
    """

    ### FIX ME ### part of catalog's motivation is to replace all this ad hoc copypasta.  TODO

    if provider_name == "cloudflare":
        from .dns_providers.cloudflare import CloudFlareDns

        CLOUDFLARE_EMAIL = os.environ.get("CLOUDFLARE_EMAIL", None)
        CLOUDFLARE_API_KEY = os.environ.get("CLOUDFLARE_API_KEY", None)
        CLOUDFLARE_TOKEN = os.environ.get("CLOUDFLARE_TOKEN", None)

        if CLOUDFLARE_EMAIL and CLOUDFLARE_API_KEY and not CLOUDFLARE_TOKEN:
            dns_class = CloudFlareDns(
                CLOUDFLARE_EMAIL=CLOUDFLARE_EMAIL,
                CLOUDFLARE_API_KEY=CLOUDFLARE_API_KEY,
                **provider_kwargs,
            )
        elif CLOUDFLARE_TOKEN and not CLOUDFLARE_EMAIL and not CLOUDFLARE_API_KEY:
            dns_class = CloudFlareDns(CLOUDFLARE_TOKEN=CLOUDFLARE_TOKEN, **provider_kwargs)
        else:
            err = (
                "ERROR:: Please supply either CLOUDFLARE_EMAIL and CLOUDFLARE_API_KEY"
                "or CLOUDFLARE_TOKEN as environment variables."
            )
            logger.error(err)
            raise KeyError(err)

    elif provider_name == "aurora":
        from .dns_providers.auroradns import AuroraDns

        try:
            AURORA_API_KEY = os.environ["AURORA_API_KEY"]
            AURORA_SECRET_KEY = os.environ["AURORA_SECRET_KEY"]

            dns_class = AuroraDns(
                AURORA_API_KEY=AURORA_API_KEY,
                AURORA_SECRET_KEY=AURORA_SECRET_KEY,
                **provider_kwargs,
            )
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "acmedns":
        from .dns_providers.acmedns import AcmeDnsDns

        try:
            ACME_DNS_API_USER = os.environ["ACME_DNS_API_USER"]
            ACME_DNS_API_KEY = os.environ["ACME_DNS_API_KEY"]
            ACME_DNS_API_BASE_URL = os.environ["ACME_DNS_API_BASE_URL"]

            dns_class = AcmeDnsDns(
                ACME_DNS_API_USER=ACME_DNS_API_USER,
                ACME_DNS_API_KEY=ACME_DNS_API_KEY,
                ACME_DNS_API_BASE_URL=ACME_DNS_API_BASE_URL,
                **provider_kwargs,
            )
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "aliyun":
        from .dns_providers.aliyundns import AliyunDns

        try:
            aliyun_ak = os.environ["ALIYUN_AK_ID"]
            aliyun_secret = os.environ["ALIYUN_AK_SECRET"]
            aliyun_endpoint = os.environ.get("ALIYUN_ENDPOINT", "cn-beijing")
            dns_class = AliyunDns(aliyun_ak, aliyun_secret, aliyun_endpoint, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "hurricane":
        from .dns_providers.hurricane import HurricaneDns

        try:
            he_username = os.environ["HURRICANE_USERNAME"]
            he_password = os.environ["HURRICANE_PASSWORD"]
            dns_class = HurricaneDns(he_username, he_password, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "rackspace":
        from .dns_providers.rackspace import RackspaceDns

        try:
            RACKSPACE_USERNAME = os.environ["RACKSPACE_USERNAME"]
            RACKSPACE_API_KEY = os.environ["RACKSPACE_API_KEY"]
            dns_class = RackspaceDns(RACKSPACE_USERNAME, RACKSPACE_API_KEY, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "dnspod":
        from .dns_providers.dnspod import DNSPodDns

        try:
            DNSPOD_ID = os.environ["DNSPOD_ID"]
            DNSPOD_API_KEY = os.environ["DNSPOD_API_KEY"]
            dns_class = DNSPodDns(DNSPOD_ID, DNSPOD_API_KEY, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "duckdns":
        from .dns_providers.duckdns import DuckDNSDns

        try:
            duckdns_token = os.environ["DUCKDNS_TOKEN"]
            dns_class = DuckDNSDns(duckdns_token=duckdns_token, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "cloudns":
        from .dns_providers.cloudns import ClouDNSDns

        try:
            dns_class = ClouDNSDns(**provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "powerdns":
        from .dns_providers.powerdns import PowerDNSDns

        try:
            powerdns_api_key = os.environ["POWERDNS_API_KEY"]
            powerdns_api_url = os.environ["POWERDNS_API_URL"]
            dns_class = PowerDNSDns(powerdns_api_key, powerdns_api_url, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "gandi":
        from .dns_providers.gandi import GandiDns

        try:
            gandi_api_key = os.environ["GANDI_API_KEY"]
            dns_class = GandiDns(GANDI_API_KEY=gandi_api_key, **provider_kwargs)
        except KeyError as e:
            logger.error("ERROR:: Please supply {0} as an environment variable.".format(str(e)))
            raise

    elif provider_name == "unbound_ssh":
        from .dns_providers.unbound_ssh import UnboundSsh

        # check & report, let calling protocol crash it.
        if "ssh_des" not in provider_kwargs:
            logger.error("ERROR: unbound_ssh REQUIRES ssh_des option.")
        dns_class = UnboundSsh(**provider_kwargs)  # pylint: disable=E1125

    elif provider_name == "route53":
        raise ValueError("route53 driver can only be used programmatically at this time, sorry")

    else:
        raise ValueError("The dns provider {0} is not recognised.".format(provider_name))

    logger.info("Using %s as registered provider.", provider_name)
    return dns_class


def main():
    "See docs/sewer-cli.md for docs & examples"

    catalog = ProviderCatalog()

    parser = setup_parser(catalog)
    args = parser.parse_args()

    loglevel = args.loglevel
    logger = lib.create_logger(None, loglevel)

    provider_name = args.provider
    domain = args.domain
    alt_domains = args.alt_domains
    if args.action != "none":
        logger.warning("DEPRECATION WARNING: --action option is obsolete and will be dropped soon")
    account_key = args.account_key
    certificate_key = args.certificate_key
    bundle_name = args.bundle_name
    endpoint = args.endpoint
    email = args.email
    out_dir = args.out_dir

    ### FIX ME ### to keep special options --domain_alias & --prop-*, or use -p_opts instead?

    provider_kwargs = {}

    for p in args.p_opts:
        parts = p.split("=")
        if len(parts) == 2:
            provider_kwargs[parts[0]] = parts[1]

    # Make sure the output dir user specified is writable
    if not os.access(out_dir, os.W_OK):
        raise OSError("The dir '{0}' is not writable".format(out_dir))

    if account_key:
        account_key = account_key.read()
    if certificate_key:
        certificate_key = certificate_key.read()
    if bundle_name:
        file_name = bundle_name
    else:
        file_name = "{0}".format(domain)
    if endpoint == "staging":
        ACME_DIRECTORY_URL = config.ACME_DIRECTORY_URL_STAGING
    else:
        ACME_DIRECTORY_URL = config.ACME_DIRECTORY_URL_PRODUCTION

    dns_class = get_provider(provider_name, provider_kwargs, catalog, logger)

    acme_client = client.Client(
        provider=dns_class,
        domain_name=domain,
        domain_alt_names=alt_domains,
        contact_email=email,
        account_key=account_key,
        certificate_key=certificate_key,
        ACME_DIRECTORY_URL=ACME_DIRECTORY_URL,
        LOG_LEVEL=loglevel,
        ACME_REQUEST_TIMEOUT=args.acme_timeout,
    )
    certificate_key = acme_client.certificate_key
    account_key = acme_client.account_key

    # prepare file path
    account_key_file_path = os.path.join(out_dir, "{0}.account.key".format(file_name))
    crt_file_path = os.path.join(out_dir, "{0}.crt".format(file_name))
    crt_key_file_path = os.path.join(out_dir, "{0}.key".format(file_name))

    # write out account_key in out_dir directory
    with open(account_key_file_path, "w") as account_file:
        account_file.write(account_key)
    logger.info("account key succesfully written to {0}.".format(account_key_file_path))

    certificate = acme_client.cert()

    # write out certificate and certificate key in out_dir directory
    with open(crt_file_path, "w") as certificate_file:
        certificate_file.write(certificate)
    with open(crt_key_file_path, "w") as certificate_key_file:
        certificate_key_file.write(certificate_key)

    logger.info("certificate succesfully written to {0}.".format(crt_file_path))
    logger.info("certificate key succesfully written to {0}.".format(crt_key_file_path))



import codecs, json, os
from setuptools import setup, find_packages

# long description comes from README.md
with codecs.open(os.path.join("docs", "README.md"), "r", encoding="utf8") as f:
    long_description = f.read()
ldct = "text/markdown"

# version and other fields in about, with envvar override
with codecs.open(os.path.join("sewer", "meta.json"), "r", encoding="utf8") as f:
    meta = json.load(f)

for k in meta:
    if "SETUP_" + k in os.environ:
        meta[k] = os.environ["SETUP_" + k]

# provider catalog, used to construct the list of extras and their deps, and all their deps
with codecs.open(os.path.join("sewer", "catalog.json"), "r", encoding="utf8") as f:
    catalog = json.load(f)

provider_deps_map = dict((i["name"], i["deps"]) for i in catalog)

all_deps_of_all_providers = list(set(sum((i["deps"] for i in catalog), [])))


setup(
    long_description=long_description,
    long_description_content_type=ldct,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Security",
        "Topic :: System :: Installation/Setup",
        "Topic :: System :: Networking",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages(exclude=["docs", "*tests*"]),
    install_requires=["requests", "pyopenssl", "cryptography"],
    extras_require=dict(
        provider_deps_map,
        dev=["coverage", "twine", "wheel"],
        test=["pylint==2.3.1", "black==18.9b0"],
        alldns=all_deps_of_all_providers,
    ),
    # data files to be placed in project directory, not zip safe but zips suck anyway
    package_data={"sewer": ["*.json"]},
    zip_safe=False,
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
    entry_points={"console_scripts": ["sewer=sewer.cli:main", "sewer-cli=sewer.cli:main"]},
    **meta,
)
